#! -*- coding: utf-8 -*-

import json
import logging
import datetime
import copy
from bson import json_util

from django.http import HttpResponseRedirect, Http404
from django.views.generic import DetailView
from django.utils.numberformat import format
from django.shortcuts import render
from django.core.paginator import Paginator
from django.shortcuts import render_to_response, RequestContext, redirect

from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.cmsutils import get_abs_url_from_slug

from acomea.fondi.models import Fund, Diary
from acomea.soap.models import AcomeaClientCms

# definitions
#acomea_client = AcomeaClientCms()
log = logging.getLogger("acomea.fondi")


class DiaryDetailView(DetailView):
    context_object_name = 'diary'
    template_name = 'bootstrap3/fondi/cms_apps/diary_detail.html'
    model = Diary
    queryset = Diary.objects.filter(status=1)

    def get(self, request, *args, **kwargs):
        try:
            self.object = self.get_object()
        except Http404:
            return redirect("/diario-del-fondo/")
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        context = super(DiaryDetailView, self).get_context_data(**kwargs)
        try:
            context.update({
                'funds': self.object.funds.all(),
            })
        except Fund.DoesNotExist:
            pass
        return context


class DiaryDetailViewModal(DiaryDetailView):
    template_name = 'fondi/cms_apps/diary_detail_modal.html'


####################
## GRAPH SETTINGS ##
####################
GRAPH_DATASET_PIE = {
    'chart': {
        'renderTo': 'container-pie',
        'type': 'pie'
    },
    'title': {
        'text': None
    },
    'subtitle': {
        'text': None
    },
    'xAxis': {
        'categories': None
    },
    'yAxis': {
        'min': 0,
        'title': {
            'text': 'Valore',
            'align': 'high'
        },
        'labels': {
            'overflow': 'justify'
        }
    },

    'plotOptions': {
        'pie': {
            'dataLabels': {
                'enabled': True
            }
        },
        'series':{
            'dataLabels': {
                'enabled': True,
                "format":'{y:.2f}'
            }
        }
    },
    'tooltip': {
        'valueDecimals': 2,
        },

    'legend': {
        'layout': 'vertical',
        'align': 'right',
        'verticalAlign': 'top',
        'x': 0,
        'y': 0,
        'floating': True,
        'borderWidth': 1,
        'backgroundColor': '#FFFFFF',
        'shadow': True
    },
    'credits': {
        'enabled': False
    },
    'series' : [{
                    'name': 'fondo',
                    'data': None,
                    'color': '#F15A22'
                }]
}


GRAPH_DATASET_BAR = {
    'chart': {
        'renderTo': 'container-bar',
        'type': 'bar'
    },
    'title': {
        'text': None
    },
    'subtitle': {
        'text': None
    },
    'xAxis': {
        'categories': None
    },
    'yAxis': {
        'min': 0,
        'title': {
            'text': 'Valore',
            'align': 'high'
        },
        'labels': {
            'overflow': 'justify'
        }
    },
    'plotOptions': {
        'bar': {
            'dataLabels': {
                'enabled': True,
            }
        },
        'series':{
            'dataLabels': {
                'enabled': True,
                "format":'{y:.2f}'
            }
        }
    },
    'tooltip': {
        'valueDecimals': 2,
    },
    'legend': {
        'layout': 'vertical',
        'align': 'right',
        'verticalAlign': 'top',
        'x': 0,
        'y': 0,
        'floating': True,
        'borderWidth': 1,
        'backgroundColor': '#FFFFFF',
        'shadow': True
    },
    'credits': {
        'enabled': False
    },
    'series' : [{
                    'name': 'fondo',
                    'data': None,
                    'color': '#F15A22'
                }]
}

GRAPH_DATASET_LINE = {
    'chart': {
        'renderTo': 'container-line',
        'type': 'line',
        'zoomType':'x',
        'width':560,
        'resetZoomButton': {
            'position': {
                'x': -15,
                'y': 15
            },
            'relativeTo': 'plot'
        }
    },
    'title': {
        'text': 'Andamento'
    },
    'subtitle': {
        'text': None
    },
    'xAxis': {
        'id':'x-axis',
        'type': 'datetime',
        'title':{
            'text':None
        },

        'dateTimeLabelFormats': {
            'month': '%b \'%y',
        },
        'tickInterval':30,
        # hide x axis - temporary (to fix) - begin
        # 'lineWidth': 0,
        # 'minorGridLineWidth': 0,
        # 'lineColor': 'transparent',
        # 'labels': {
        #     'enabled': 'false'
        # },
        # 'minorTickLength': 0,
        # 'tickLength': 0
        # hide x axys - temporary (to fix) - end
    },
    'yAxis': {
        'title': {
            'text': 'Valore',
        },
        'min':None,
        'max':None
    },
    'tooltip': {
        'crosshairs':True,
        'shared':True
    },
    'plotOptions': {
        'line': {
            'lineWidth':1,
            'marker': {
                'enabled': False
            },
            'shadow':False
        }
    },
    # 'series' : {
    #      'events': {
    #                 'hide': 'function(a) { this.xAxis.hide(); }',
    #                 'show': 'function() { this.xAxis.show();  }'
    #      }
    # },
    # 'legend': {
    #     'layout': 'vertical',
    #     'align': 'right',
    #     'verticalAlign': 'top',
    #     'x': 0,
    #     'y': 0,
    #     'floating': True,
    #     'borderWidth': 1,
    #     'backgroundColor': '#FFFFFF',
    #     'shadow': True
    # },
    'credits': {
        'enabled': False
    },
    # 'series' : [{
    #                 'name': 'fondo',
    #                 'data': None,
    #                 'color': '#F15A22'
    #             }]
}


#########################
## FUND CMS APPS VIEWS ##
#########################
class FundDetailView(DetailView):
    """
    base fund detail view
    """

    context_object_name = 'fondo'
    template_name = 'fondi/cms_apps/fund_detail.html'
    model = Fund

    def __init__(self):
        self.acomea_client = AcomeaClientCms()
        super(FundDetailView, self).__init__()

    def get_object(self, queryset=None):
        try:
            return Fund.objects.get(slug=self.kwargs['slug'])
        except Fund.DoesNotExist:
            raise ForceResponse(HttpResponseRedirect("/404"))

    def get_context_data(self, **kwargs):
        context_data = super(FundDetailView, self).get_context_data(**kwargs)
        context_data.update({'buy_product_url':get_abs_url_from_slug('investi-disinvesti')})
        return context_data

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return render(request, self.template_name, context)


class ModalitaPartecipazioneView(FundDetailView):
    template_name = "fondi/cms_apps/modalita_partecipazione.html"


class SpeseCommissioniView(FundDetailView):
    template_name = "fondi/cms_apps/spese_commissioni.html"


class FundAnalysisView(FundDetailView):
    template_name = 'fondi/cms_apps/fund_analysis.html'

    def get_data_set(self):
        try:
            data = self.get_data_from_soap()
            fund = Fund.objects.get(code=self.object.code)
            dst = None
            data_set = {}

            dsb = GRAPH_DATASET_BAR
            dsb['title']['text'] = data['line']['Titolo']
            dsb['subtitle']['text'] = data['line']['SottoTitolo']
            dsb['xAxis']['categories'] = data['line']['Categories']
            dsb['series'][0]['data'] = data['line']['SeriesData']
            dsb['plotOptions']['series']['dataLabels']['format'] = '{y:.2f}'
            dst =  data['line']['SeriesData']
            data_set['bar'] = json.dumps(dsb, default=json_util.default)

            dsp = GRAPH_DATASET_PIE
            dsp['title']['text'] = data['pie']['Titolo']
            dsp['subtitle']['text'] = data['pie']['SottoTitolo']
            dsp['xAxis']['categories'] = data['pie']['Categories']
            dsp['series'][0]['data'] = data['pie']['SeriesData']
            dst =  data['pie']['SeriesData']
            data_set['pie'] = json.dumps(dsp, default=json_util.default)
            data_set['fund'] = fund

            data_set['table'] = dst
            data_set['last_update']=data['line']['DataAggiornamento']
            data_set['active_buttons']=data['active_buttons']
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            data_set = {}

        return data_set

    def get_data_from_soap(self):
        return {}

    def get_context_data(self, **kwargs):

        context = super(FundAnalysisView, self).get_context_data(**kwargs)
        context['data_set']=self.get_data_set()
        context['fund_analysis_child'] = True
        return context


class FundYieldsView(FundDetailView):
    template_name = 'fondi/cms_apps/fund_yields.html'

    def get_data_set(self, **kwargs):
        return {}

    def get_context_data(self, **kwargs):
        context = super(FundYieldsView, self).get_context_data(**kwargs)
        context['data_set']=self.get_data_set(**kwargs)
        context['fund_yields_child'] = True
        context['classe']='A1'
        if 'classe' in self.kwargs:
            context['classe']=self.kwargs['classe']
        return context


# nuove viste (CBV): da vistesoap a fondi
class AndamentoView(FundYieldsView):
    template_name = 'fondi/cms_apps/_andamento.html'

    def get_data_set(self, **kwargs):
        if 'classe' in self.kwargs:
            classe = self.kwargs['classe']
        else:
            classe = 'A1'

        try:
            data = self.acomea_client.get_andamento(self.object.code, classe)
            # FIXME: workaround temporaneo (il filtro 'intcomma' non funziona correttamente)
            patrimonio_formatted = format(data['NAV'], decimal_sep=",", decimal_pos=2,
                                          grouping=3, thousand_sep=".", force_grouping=True)
            data['NAV'] = patrimonio_formatted
        except Exception, e:
            log.error("Exc in AndamentoView : %s" % str(e.message))
            data = {}

        return data


class SerieStoricaView(FundYieldsView):
    template_name = 'fondi/cms_apps/_serie_storica.html'

    def get_data_set(self, **kwargs):
        if 'classe' in self.kwargs:
            classe = self.kwargs['classe']
        else:
            classe = 'A1'

        try:
            data_riferimento = None
            if self.request.GET:
                if "data_rif" in self.request.GET:
                    try:
                        data_riferimento = datetime.datetime.strptime(self.request.GET['data_rif'], "%d-%m-%Y")
                    except:
                        pass
            data = self.acomea_client.get_serie_storica(self.object.code, classe, data_riferimento)

        except Exception, e:
            log.error("Exc in SerieStoricaView : %s" % str(e.message))
            data = {}
        return data


class RendimentiAnnuiView(FundYieldsView):
    template_name = "fondi/cms_apps/_rendimenti_annui.html"

    def get_data_set(self, **kwargs):
        if 'classe' in self.kwargs:
            classe = self.kwargs['classe']
        else:
            classe = 'A1'

        try:
            data = self.acomea_client.get_rendimenti_annui(self.object.code, classe)
        except Exception, e:
            log.error("Exc in RendimentiAnnuiView : %s" % str(e.message))
            data = {}
        return data


class StacchiCedolaView(FundYieldsView):
    template_name = "fondi/cms_apps/_rendimenti_annui.html"

    def get_data_set(self, **kwargs):
        data = {}
        if 'classe' in self.kwargs:
            classe = self.kwargs['classe']

        if classe == 'C1' or classe == 'C2':
            try:
                data = self.acomea_client.get_stacchi_cedola(self.object.code, classe)
            except Exception, e:
                log.error("Exc in StacchiCedolaView : %s" % str(e.message))
        return data



class GraficoView(FundYieldsView):
    template_name = "fondi/cms_apps/_grafico.html"

    def get_data_set(self, **kwargs):
        if 'classe' in self.kwargs:
            classe = self.kwargs['classe']
        else:
            classe = 'A1'

        try:
            data_dal = data_al = None
            if self.request.GET:
                if "data_dal" in self.request.GET:
                    data_dal = datetime.datetime.strptime(self.request.GET['data_dal'], "%d-%m-%Y")
                if "data_al" in self.request.GET:
                    data_al = datetime.datetime.strptime(self.request.GET['data_al'], "%d-%m-%Y")
            if not data_dal:
                data_dal = datetime.datetime.now() - datetime.timedelta(18*365/12)
            if not data_al:
                data_al = datetime.datetime.now()

            raw_data = self.acomea_client.get_grafico(self.object.code, classe, data_dal, data_al)
            dsl = copy.deepcopy(GRAPH_DATASET_LINE)
            dsl['series']=[]
            serie_fondo = {
                'name': 'Valore Fondo',
                'data' : [],
                'color': '#3E576F'
            }
            serie_benchmark = {
                'name': 'Valore Benchmark',
                'data':[],
                'color': '#AA4643'
            }

            first_item = raw_data['SeriesData'][0]
            last_item = raw_data['SeriesData'][-1]
            ymin = ymax = float(first_item['valf'])
            months_dict = {}
            for l in raw_data['SeriesData']:
                datepy = datetime.date(int(l['data'][0:4]), int(l['data'][5:7]), int(l['data'][8:10]))
                serie_fondo['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valf'])))
                serie_benchmark['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valb'])))
                if float(l['valf']) < ymin:
                    ymin = float(l['valf'])
                if float(l['valf']) > ymax:
                    ymax = float(l['valf'])
                mm_yyyy = datepy.strftime('%m %Y')
                if mm_yyyy not in months_dict:
                    months_dict[mm_yyyy]=datepy
            first_date = datetime.date(int(first_item['data'][0:4]),
                                       int(first_item['data'][5:7]),
                                       int(first_item['data'][8:10]))
            last_date = datetime.date(int(last_item['data'][0:4]),
                                      int(last_item['data'][5:7]),
                                      int(last_item['data'][8:10]))
            dsl['yAxis']['min'] = ymin - 0.15 # yAxis lower bound
            dsl['yAxis']['max'] = ymax + 1.50 # yAxis upper bound

            months_list = list(months_dict.values())
            months_list.sort()
            months_list_str = []
            for m in months_list:
                months_list_str.append(m.strftime("%b %y"))

            dsl['xAxis']['categories'] = months_list_str
            dsl['series'].append(serie_fondo)
            if raw_data['HasBenchmark']:
                dsl['series'].append(serie_benchmark)

            # dsl['series'][0]['pointStart'] = "Date.UTC("+str(first_date.year)+"," \
            #                               " "+str(first_date.month-1)+", "+str(first_date.day)+")"
            # dsl['series'][0]['pointInterval'] = 30*24*3600*1000 # one month (in ms)
            data_set = {}
            data_set['data_dal'] = data_dal
            data_set['data_al'] = data_al
            data_set['classe'] = classe
            data_set['raw'] = raw_data
            data_set['line'] = json.dumps(dsl, default=json_util.default)

        except Exception, e:
            log.error("Exc in GraficoView : %s" % str(e.message))
            data_set = {}

        return data_set


class ClassiAttivitaView(FundAnalysisView):
    template_name = "fondi/cms_apps/_classi_di_attivita.html"

    def get_data_from_soap(self, **kwargs):
        try:
            data = self.acomea_client.get_classi_di_attivita(self.object.code)
#            data = {
#                'line': {
#                    'DataAggiornamento': datetime.datetime(2013, 3, 13, 0, 0),
#                    'IDTAB': u'CP3',
#                    'Precalcolata': True,
#                    'Colonne': 0L,
#                    'Ordine': 1L,
#                    'ID': u'CP', 'TitoloBreve': u'Classi di attivit\xe0',
#                    'SeriesData': [0.124, 41.9087, 0.5002, 0.4958, 56.844, 0.0],
#                    'SottoTitolo': u'Dati in % calcolati sul Nav del fondo. ',
#                    'Righe': 0L, 'Titolo': u'Classi di attivit\xe0',
#                    'Categories': [
#                        'Azioni/Warrants', 'Titoli di Stato', 'ETC', 'Obbl. Convertibili',
#                        'Obbl. Soc. e Altri Emitt.', 'Liquidita', 'Azioni/Warrants', 'Titoli di Stato',
#                        'ETC', 'Obbl. Convertibili', 'Obbl. Soc. e Altri Emitt.', 'Liquidita'
#                    ]
#                }, 'pie': {
#                    'DataAggiornamento': datetime.datetime(2013, 3, 13, 0, 0),
#                    'IDTAB': u'CP3', 'Precalcolata': True, 'Colonne': 0L, 'Ordine': 1L, 'ID': u'CP',
#                    'TitoloBreve': u'Classi di attivit\xe0', 'SeriesData': [
#                        ['Azioni/Warrants', 0.124], ['Titoli di Stato', 41.9087],
#                        ['ETC', 0.5002], ['Obbl. Convertibili', 0.4958],
#                        ['Obbl. Soc. e Altri Emitt.', 56.844], ['Liquidita', 0.0]
#                    ],
#                    'SottoTitolo': u'Dati in % calcolati sul Nav del fondo. ',
#                    'Righe': 0L, 'Titolo': u'Classi di attivit\xe0',
#                    'Categories': [
#                        'Azioni/Warrants', 'Titoli di Stato', 'ETC', 'Obbl. Convertibili',
#                        'Obbl. Soc. e Altri Emitt.', 'Liquidita', 'Azioni/Warrants',
#                        'Titoli di Stato', 'ETC', 'Obbl. Convertibili', 'Obbl. Soc. e Altri Emitt.',
#                        'Liquidita']
#                }
#            }
            return data
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class TitoliView(FundAnalysisView):
    template_name = "fondi/cms_apps/_titoli.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_titoli(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class RatingView(FundAnalysisView):
    template_name = "fondi/cms_apps/_rating.html"

    def get_data_from_soap(self, **kwargs):

        try:
            return self.acomea_client.get_rating(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class ValuteView(FundAnalysisView):
    template_name = "fondi/cms_apps/_valute.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_valute(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class PrincipaliPaesiView(FundAnalysisView):
    template_name = "fondi/cms_apps/_principali_paesi.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_principali_paesi(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class ScadenzaView(FundAnalysisView):
    template_name = "fondi/cms_apps/_scadenza.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_scadenza(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


class AreeGeograficheView(FundAnalysisView):
    template_name = "fondi/cms_apps/_aree_geografiche.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_aree_geografiche(self.object.code)
        except Exception, e:
            log.error("Exc in " + self.__class__.__name__ + ": %s" % str(e.message))
            return {}

    def get_data_set(self):
        try:
            data = self.get_data_from_soap()
            fund = Fund.objects.get(code=self.object.code)
            dst = None
            data_set = {}

            dsb = GRAPH_DATASET_BAR
            dsb['title']['text'] = data['line']['Titolo']
            dsb['subtitle']['text'] = data['line']['SottoTitolo']
            dsb['xAxis']['categories'] = data['line']['Categories']
            dsb['series'][0]['data'] = data['line']['SeriesData']
            dsb['plotOptions']['series']['dataLabels']['format'] = '{y:.2f}'
            dst = data['line']['SeriesData']
            data_set['bar'] = json.dumps(dsb, default=json_util.default)

            dsp = GRAPH_DATASET_PIE
            dsp['title']['text'] = data['pie']['Titolo']
            dsp['subtitle']['text'] = data['pie']['SottoTitolo']
            dsp['xAxis']['categories'] = data['pie']['Categories']
            dsp['series'][0]['data'] = data['pie']['SeriesData']
            dst = data['pie']['SeriesData']
            data_set['pie'] = json.dumps(dsp, default=json_util.default)
            data_set['fund'] = fund

            data_set['table'] = data['table']
            data_set['last_update'] = data['line']['DataAggiornamento']
            data_set['active_buttons'] = data['active_buttons']
        except Exception, e:
            # log.error("Exc in "+self.__class__.__name__+": %s" % str(e))
            print("Exc in " + self.__class__.__name__ + ": %s" % str(e.message))
            data_set = {}

        return data_set


class SettoriView(FundAnalysisView):
    template_name = "fondi/cms_apps/_settori.html"

    def get_data_from_soap(self, **kwargs):
        try:
            return self.acomea_client.get_settori(self.object.code)
        except Exception, e:
            log.error("Exc in "+self.__class__.__name__+": %s" % str(e.message))
            return {}


def ajax_fund_diary_load(request):
    try:
        if request.is_ajax():
            page = int(request.GET.get('page', 1))
            fund_id = request.GET.get('fund_id', None)
            fund = Fund.objects.get(id=fund_id) if fund_id else None
            adjacent_pages = 3
            objects_per_page = 5
            template = 'bootstrap3/fondi/cms_plugins/fund_diary_template.html'

            if fund:
                 # Paginator
                paginator = Paginator(fund.diaries.all(), objects_per_page)
                objects = paginator.page(page)
                page_numbers = [n for n in range(page-adjacent_pages, page+adjacent_pages+1) if n>0 and n<=paginator.num_pages]
                show_first = True if 1 not in page_numbers else False
                show_last = True if paginator.num_pages not in page_numbers else False
                context = {
                    'prev_page': page-1,
                    'next_page': page+1,
                    'objects': objects,
                    'adjacent_pages': adjacent_pages,
                    'page_numbers': page_numbers,
                    'show_first': show_first,
                    'show_last': show_last,
                }
                return render_to_response(template, context, context_instance = RequestContext(request))
    except Exception, e:
        print e