# -*- coding: utf-8 -*-
import logging
import datetime, calendar, rfc822
import twitter

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_extensions.db.fields import AutoSlugField

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.core.db.fields import HTMLFieldTinyMCE
from filer.fields.image import FilerImageField , AdminImageWidget
from image_cropping import ImageRatioField

log = logging.getLogger("acomea.fondi")


class CategoryArea(DateModel, UserModel, StatusModel, OrderedModel):
    name = models.CharField(_("Name"), max_length=32, unique=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _("Category area")
        verbose_name_plural = _("Category areas")


class FundCategory(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        _("Code"), max_length=2, unique=True)
    title = models.CharField(_("Title"), max_length=50)
    description = HTMLFieldTinyMCE(_("description"), blank=True, null=True)
    slug = models.SlugField(
        _("Slug"),
    )
    categories_area = models.ForeignKey(
        CategoryArea,
        blank=True, null=True
    )

    def __unicode__(self):
        return self.code

    class Meta:
        verbose_name = _("Fund category")
        verbose_name_plural = _("Fund categories")
        ordering = ("ordering",)

    #        def num_box(self):
    #            return range(0, 8 - self.fondo_set.count())

    def get_title(self):
        """
        sostituisce lo spazio con un a capo.
        utilizzato per i box nel sito
        """
        return self.title.replace(" ", "<br />", 1)

    def get_funds(self):
        return self.funds.filter(status=1).order_by('ordering', 'title')


class Fund(DateModel, UserModel, StatusModel, OrderedModel, SEOModel):
    code = models.CharField(
        _("Code"),
        max_length=2,
        unique=True)
    fund_category = models.ForeignKey(FundCategory, related_name='funds')
    title = models.CharField(
        _("Title"),
        max_length=50
    )
    slug = models.SlugField(
        _("Slug"),
    )
    assogestioni_category = models.CharField(
        _("Categoria Assogestioni"),
        max_length=200,
    )
    short_description = models.CharField(
        _("Short description"),
        max_length=256,
        blank=True, null=True
    )
    long_description = HTMLFieldTinyMCE(
        _("Long Description"),
        blank=True, null=True
    )
    detail_page_url = models.CharField(
        _("Fund detail page slug"),
        max_length=150, blank=True,
    )
    currency = models.CharField(
        max_length=1,
        blank=True, null=True,
        choices=settings.FONDI_FUND_CURRENCY_CHOICES,
        verbose_name=_('currency')
    )
    benchmark = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('Benchmark')
    )
    scope = models.TextField(
        blank=True, null=True,
        verbose_name=_('Scope')
    )
    isin_a1_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN A1 Class')
    )
    isin_a2_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN A2 Class')
    )
    isin_q2_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN Q2 Class')
    )
    isin_p1_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN P1 Class')
    )
    isin_p2_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN P2 Class')
    )
    isin_c1_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN C1 Class')
    )
    isin_c2_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN C2 Class')
    )
    isin_a5_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN A5 Class')
    )
    isin_p5_class = models.CharField(
        max_length=25,
        blank=True, null=True,
        verbose_name=_('ISIN P5 Class')
    )
    expense_a1_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense A1 % '),
    )
    expense_a2_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense A2 % '),
    )
    expense_q2_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense Q2 % '),
    )
    expense_p1_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense P1 % '),
    )
    expense_p2_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense P2 % '),
    )
    expense_c1_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense C1 % '),
    )
    expense_c2_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense C2 % '),
    )
    expense_a5_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense A5 % '),
    )
    expense_p5_class = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        null=True,
        blank=True,
        verbose_name=_('expense P5 % '),
    )
    risk_profile = models.IntegerField(
        null=True,
        blank=True,
        verbose_name=_('risk profile'),
    )
    promoters = models.ManyToManyField(
        'Promoter',
        blank=True, null=True,
        related_name='funds',
        verbose_name=_('promoters')
    )
    copromoters = models.ManyToManyField(
        'Promoter',
        blank=True, null=True,
        related_name='copromoter_funds',
        verbose_name=_('copromoters')
    )
    awards = models.ManyToManyField(
        'Award',
        blank=True, null=True,
        related_name='funds',
        verbose_name=_('awards')
    )
    document_path = models.CharField(
        _("Document path"),
        max_length=512, blank=True, null=True,
        help_text=_("path to the folder that contains the documentation files")
    )
    twitter_id = models.CharField(
        _("Twitter ID"),
        max_length=100,
        blank=True, null=True
    )
    analysis_header_text = HTMLFieldTinyMCE(
        _("Analysis header text"),
        blank=True, null=True,
    )
    analysis_footer_text = HTMLFieldTinyMCE(
        _("analysis footer text"),
        blank=True, null=True,
    )
    yields_header_text = HTMLFieldTinyMCE(
        _("Yields header text"),
        blank=True, null=True,
    )
    yields_footer_text = HTMLFieldTinyMCE(
        _("Yields footer text"),
        blank=True, null=True,
    )
    classes_analysis_representation = models.CharField(
        _("Classes analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    title_analysis_representation = models.CharField(
        _("Title analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    rating_analysis_representation = models.CharField(
        _("Rating analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    countries_analysis_representation = models.CharField(
        _("Countries analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    deadline_analysis_representation = models.CharField(
        _("Deadline analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    duration_analysis_representation = models.CharField(
        _("Duration analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    sectors_analysis_representation = models.CharField(
        _("Sectors analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    currencies_analysis_representation = models.CharField(
        _("Currencies analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )
    areas_analysis_representation = models.CharField(
        _("Areas analysis representation"),
        max_length=16,
        choices=settings.FONDI_CHOICES_REPRESENTATION
    )

    def __unicode__(self):
        return "%s - %s" % (self.code, self.title)

    def performance_and_share(self):
        return _("rendimenti e quota di %s") % self.code

    def get_portfolio_analysis(self):
        return _("analisi del portafoglio di %s") % self.code

    def get_diaries(self, future=False):
        qs = self.diaries.filter(status=1)
        if not future:
            qs = qs.filter(date__lte=datetime.date.today())
        return qs.order_by('-date')

    def get_slug_detail_page(self):
        return "/scopri-prodotti/fondi/%s/%s" % (self.fund_category.slug, self.slug)

    def get_num_classi(self):
        num = 0
        if self.isin_a1_class:
            num += 1
        if self.isin_a2_class:
            num += 1
        if self.isin_q2_class:
            num += 1
        if self.isin_p1_class:
            num += 1
        if self.isin_p2_class:
            num += 1
        if self.isin_c1_class:
            num += 1
        if self.isin_c2_class:
            num += 1
        if self.isin_a5_class:
            num += 1
        if self.isin_p5_class:
            num += 1
        return num

    class Meta:
        verbose_name = _("Fund")
        verbose_name_plural = _("Funds")
        ordering = ('fund_category__title', 'title')


class Promoter(DateModel, UserModel, StatusModel, OrderedModel):
    name = models.CharField(
        max_length=255,
        verbose_name=_("name")
    )
    image = FilerImageField(
        blank=True, null=True,
        verbose_name=_('image'),
    )
    description = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    short_description = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('short description'),
    )

    class Meta:
        verbose_name = _("Promoter")
        verbose_name_plural = _("Promoters")
        ordering = ('name',)

    def __unicode__(self):
        return self.name


class Award(DateModel, UserModel, StatusModel, OrderedModel):
    title = models.CharField(
        max_length=255,
        verbose_name=_("title")
    )
    date_awd = models.DateField(
        verbose_name=_('award date')
    )
    image = FilerImageField(
        verbose_name=_('image'),
    )
    url = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('url')
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )

    class Meta:
        verbose_name = _("Award")
        verbose_name_plural = _("Awards")
        ordering = ('date_awd',)

    def __unicode__(self):
        return self.title


class CachedTweet(DateModel, UserModel, StatusModel, OrderedModel):
    testo = models.CharField(
        _("testo"),
        max_length=240
    )
    data = models.DateTimeField(blank=True, null=True)
    fund = models.ForeignKey(Fund)


### snippet for integrate filer with django-cropping ###

from filer import settings as filer_settings
from filer.fields.image import (
    AdminImageWidget, AdminImageFormField, FilerImageField,
)
from filer.models import File


class CroppableImageWidget(AdminImageWidget):
    def render(self, name, value, attrs=None):
        if value:
            file_obj = File.objects.get(pk=value)
            attrs = attrs or {}
            attrs.update({
                'class': 'crop-thumb',
                'data-thumbnail-url':
                    file_obj.thumbnails['admin_sidebar_preview'],
                'data-field-name': name,
                'data-org-width': file_obj.width,
                'data-org-height': file_obj.height,
            })

        return super(CroppableImageWidget, self).render(name, value, attrs)

    class Media:
        js = (
            filer_settings.FILER_STATICMEDIA_PREFIX + 'js/popup_handling.js',
            getattr(settings, 'JQUERY_URL',
                    'https://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js'),
            "image_cropping/js/jquery.Jcrop.min.js",
            "image_cropping/image_cropping.js",
        )
        css = {'all': ("image_cropping/css/jquery.Jcrop.min.css",)}


class CroppableFormField(AdminImageFormField):
    widget = CroppableImageWidget


class CroppableFilerImageField(FilerImageField):
    default_form_class = CroppableFormField


class Diary(DateModel, UserModel, StatusModel, OrderedModel, SEOModel):
    title = models.CharField(_("Title"), max_length=70)
    slug = models.SlugField(
        _("slug"),
        unique=True,
    )
    funds = models.ManyToManyField(
        Fund,
        related_name="diaries",
        verbose_name=_("funds"),
    )
    description = models.CharField(
        _("Description"),
        max_length=100,
        blank=True, null=True
    )
    text = HTMLFieldTinyMCE(
        _("Text"),
        blank=True, null=True
    )
    date = models.DateField(
        _("Date"),
        blank=True, null=True,
        default=datetime.date.today,
        help_text=_("date nel futuro si possono mettere e verranno visualizzate allo scadere.")
    )
    thumbnail = CroppableFilerImageField(
        blank=True, null=True,
        verbose_name=_('thumbnail'),
    )
    cropping = ImageRatioField(
        'thumbnail',
        '1200x628',
        size_warning=True,
        verbose_name=_('cropping'),
    )

    tweet = models.CharField(
        _("Tweet"),
        max_length=140,
        blank=True, null=True,
        help_text="se lasciato vuoto non tweetto nulla, \
                   la parola 'URL' diventa la url del dettaglio del \
                   diario stesso."
    )
    hide = models.BooleanField(
        _("Keep hide"),
        default=False,
        help_text="se selezionato il diario non\
                   comparirà nella pagina del fondo. (di solito\
                   perchè abbiamo il link al dettaglio nel \
                   tweet)"
    )

    def get_fund_list(self):
        list = []
        for fund in self.funds.all():
            list.append(fund.__unicode__())
        return list

    def get_fund_string(self):
        return ", ".join(self.get_fund_list())
    get_fund_string.short_description = _('funds')

    def get_tweet(self):
        output = self.tweet
        start_url_position = self.tweet.find("http")
        if start_url_position > -1:
            end_url_position = self.tweet.find(" ", start_url_position)
            if end_url_position == -1:
                end_url_position = len(self.tweet)
            if self.tweet.find("https") > -1:
                http_length = 8
            else:
                http_length = 7
            output = output.replace(
                self.tweet[start_url_position:end_url_position],
                "<a href='%s' target='_blank'>%s</a>" %
                (self.tweet[start_url_position:end_url_position],
                 self.tweet[start_url_position + http_length:end_url_position]))
        url_pos = output.find("URL")
        if url_pos != -1:
            output = "%s%s%s" % (output[0:url_pos],
                                 "<a href='http://www.acomea.it%s'>%s</a>" %
                                 (reverse('dettaglio_diario', args=(self.slug,)), self.slug),
                                 output[url_pos + 3:])
        return output

    def save(self, *args, **kwargs):
        self.title = self.title.strip()
        if self.tweet != None and self.pk == None:
            super(Diary, self).save(*args, **kwargs)
            try:
                twit = self.tweet.replace(u'è', '&egrave').replace(u'à', '&agrave').replace(u'ì', '&igrave').replace(
                    u'ò', '&ograve').replace(u'ù', '&ugrave').replace(u'£', '&pound').replace(u'€', '&#8364').replace(
                    u'°', '&deg')
                #twit.unicode()
                #decoder = codecs.getdecoder("utf-8")
                #twit,_ = decoder(self.tweet)
                tokens_dict = settings.FONDI_OAUTHTOKENS[self.fund.code]
                oauth = twitter.OAuth(
                    tokens_dict['token'],
                    tokens_dict['token_secret'],
                    tokens_dict['consumer_key'],
                    tokens_dict['consumer_secret'])
                t = twitter.Twitter(auth=oauth)
                url_pos = twit.find("URL")
                if url_pos != -1:
                    status = "%s%s%s" % (
                        twit[0:url_pos],
                        "http://www.acomea.it%s" %
                        reverse('dettaglio_diario', args=(self.slug,)),
                        twit[url_pos + 3:]
                    )
                    status = str(status)
                else:
                    status = twit
                print "HERE"
                t.statuses.update(status=status)
            except Exception, e:
                log.warning("Diary : exception trying to tweet : %s" % str(e))
        else:
            super(Diary, self).save(*args, **kwargs)

    def __unicode__(self):
        return "%s - %s" % (self.get_fund_string(), self.title)

    class Meta:
        verbose_name = _("Diary")
        verbose_name_plural = _("Diaries")
        ordering = ('funds__title', '-date',)


def load_tweets(fondi=Fund.objects.all()):
    """Da mettere in un command sotto crontab per caricare localmente i tweet dei fondi."""
    try:
        tweets_fondi = []
        for f in fondi:
            try:
                tw_api = twitter.Twitter()
                tweets = tw_api.statuses.user_timeline(id=f.twitter_id, count=2,
                                                       include_entities=True)
                for tweet in tweets:

                    tweet_acomea_testo = tweet['text']
                    tweet_acomea_fund = f
                    for url in tweet['entities']['urls']:
                        if 'display_url' in url:
                            tweet_acomea_testo = tweet_acomea_testo.replace(
                                url['url'], "<a href='%s' target='_blank'>%s</a>" % (url['url'], url['display_url'],)
                            )
                    if 'media' in tweet['entities']:
                        tweet_acomea_testo = tweet_acomea_testo.replace(
                            tweet['entities']['media'][0]['url'],
                            (
                                "<img src='%s'/ width='210px'>"
                            ) % tweet['entities']['media'][0]['media_url_https']
                        )
                    tweet_acomea, created = CachedTweet.objects.get_or_create(
                        testo=tweet_acomea_testo, fund=tweet_acomea_fund
                    )
                    if created:
                        tweet_acomea.data = datetime.datetime.fromtimestamp(
                            calendar.timegm(rfc822.parsedate(tweet['created_at']))
                        )
                        tweet_acomea.save()
                        tweets_fondi.append(tweet_acomea)
            except Exception, e:
                log.warning("Exception in load_tweets for %s : %s" % (f, str(e)))
        return tweets_fondi
    except Exception, e:
        log.warning("Exception in load_tweets : %s" % str(e))


if settings.CMS_ENABLED:
    import acomea.fondi.cms.plugins
    import acomea.fondi.cms.app